@@ -253,6 +253,11 @@ struct TiledInputFile::Data: public Mutex
     Data (Data&& other) = delete;
     Data& operator = (Data&& other) = delete;
 
+    static const int gLargeChunkTableSize = 1024*1024;
+    void validateStreamSize();                       // throw an exception if the file is significantly
+                                                     // smaller than the data/tile geometry would require
+
+
     inline TileBuffer * getTileBuffer (int number);
 					    // hash function from tile indices
 					    // into our vector of tile buffers
@@ -299,6 +304,58 @@ TiledInputFile::Data::getTileBuffer (int number)
 }
 
 
+
+//
+// avoid allocating excessive memory due to large lineOffsets table size.
+// If the chunktablesize claims to be large,
+// check the file is big enough to contain the table before allocating memory
+// in the bytesPerLineTable and the lineOffsets table.
+// Attempt to read the last entry in the first level of the table. Either the seekg() or the read()
+// call will throw an exception if the file is much too small to contain the table.
+//
+
+// assumes the input stream pointer is at (or before) the beginning of the chunk table
+
+
+void
+TiledInputFile::Data::validateStreamSize()
+{
+    const TileDescription& td = header.tileDescription();
+    Int64 chunkCount;
+
+    if (td.mode==RIPMAP_LEVELS)
+    {
+        // use slow function to calculate exact size of ripmap
+        chunkCount = getTiledChunkOffsetTableSize(header);
+    }
+    else
+    {
+        // for ONE_LEVEL image, calculate exact number of tiles
+        // MIPMAP_LEVELS images will have roughly 1/3 more tiles than this
+        // but 'chunkCount' can be less than the real offset table size for a meaningful sanity check
+        //
+        const Box2i &dataWindow = header.dataWindow();
+        Int64 tileWidth = td.xSize;
+        Int64 tileHeight = td.ySize;
+
+        Int64 tilesX = (static_cast<Int64>(dataWindow.max.x+1-dataWindow.min.x) + tileWidth -1) / tileWidth;
+        Int64 tilesY = (static_cast<Int64>(dataWindow.max.y+1-dataWindow.min.y) + tileHeight -1) / tileHeight;
+
+        chunkCount = tilesX*tilesY;
+    }
+
+    if (chunkCount > gLargeChunkTableSize)
+    {
+
+       Int64 pos = _streamData->is->tellg();
+       _streamData->is->seekg(pos + (chunkCount-1)*sizeof(Int64));
+       Int64 temp;
+       OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::read <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (*_streamData->is, temp);
+       _streamData->is->seekg(pos);
+    }
+
+}
+
 namespace {
 
 void
@@ -609,10 +666,11 @@ TileBufferTask::execute ()
                     // The frame buffer contains a slice for this channel.
                     //
     
-                    char *writePtr = slice.base +
+                    intptr_t base = reinterpret_cast<intptr_t>(slice.base);
+                    char *writePtr = reinterpret_cast<char*>(base +
                                      (y - yOffset) * slice.yStride +
                                      (tileRange.min.x - xOffset) *
-                                     slice.xStride;
+                                     slice.xStride);
 
                     char *endPtr = writePtr +
                                    (numPixelsPerScanLine - 1) * slice.xStride;
@@ -731,6 +789,7 @@ TiledInputFile::TiledInputFile (const char fileName[], int numThreads):
             _data->_streamData = new InputStreamMutex();
             _data->_streamData->is = is;
             _data->header.readFrom (*_data->_streamData->is, _data->version);
+
             initialize();
             //read tile offsets - we are not multipart or deep
             _data->tileOffsets.readFrom (*(_data->_streamData->is), _data->fileIsComplete,false,false);
@@ -958,7 +1017,10 @@ TiledInputFile::initialize ()
     {
         if (!isTiled (_data->version))
             throw IEX_NAMESPACE::ArgExc ("Expected a tiled file but the file is not tiled.");
-        
+
+        if (isNonImage (_data->version))
+            throw IEX_NAMESPACE::ArgExc ("File is not a regular tiled image.");
+
     }
     else
     {
@@ -970,6 +1032,15 @@ TiledInputFile::initialize ()
     
     _data->header.sanityCheck (true);
 
+    //
+    // before allocating memory for tile offsets, confirm file is large enough
+    // to contain tile offset table
+    // (for multipart files, the chunk offset table has already been read)
+    //
+    if (!isMultiPart(_data->version))
+    {
+        _data->validateStreamSize();
+    }
     _data->tileDesc = _data->header.tileDescription();
     _data->lineOrder = _data->header.lineOrder();
 
@@ -999,6 +1070,16 @@ TiledInputFile::initialize ()
 
     _data->tileBufferSize = _data->maxBytesPerTileLine * _data->tileDesc.ySize;
 
+    //
+    // OpenEXR has a limit of INT_MAX compressed bytes per tile
+    // disallow uncompressed tile sizes above INT_MAX too to guarantee file is written
+    //
+    if( _data->tileBufferSize > INT_MAX )
+    {
+        throw IEX_NAMESPACE::ArgExc ("Tile size too large for OpenEXR format");
+    }
+
+
     //
     // Create all the TileBuffers and allocate their internal buffers
     //
